    processor 6502
    include vcs.h
    include sleep.h
    include macro.h

;FPS = 50 or 60
;PAL = 0 or 1
#if FPS==50         ;PAL
VBLNK   equ 48
LINES   equ 228
OVERSCN equ 36
#else               ;NTSC
VBLNK   equ 40
LINES   equ 192
OVERSCN equ 30
#endif

    ;RAM
    SEG.U VARS
    org $80
    
frame ds    1

    echo "RAM:", ($100 - *)d, "bytes left"

    ;ROM
    SEG CODE
#if TWOK
    ;needed for Harmony cart
    ;it seems its firmware can only do mirroring for 2k ROMs, not 1k and below
    org $1800
#else
    org $1F80
#endif

    ;first, let's have some beautiful graphics
Gfx0
    byte %01010101
    byte %10010101
    byte %10011101
    byte %10010101
    byte %01001001

Gfx1
    byte %01001101
    byte %01000100
    byte %01000100
    byte %11000100
    byte %01000100

Gfx2
    byte %01001000
    byte %01010101
    byte %11010101
    byte %01010101
    byte %01001001

Gfx3
    byte %10100101
    byte %00001010
    byte %10101000
    byte %10101000
    byte %10101000

    ;here comes the code
Start
                ;CLEAN_START
                ;sei    ;6507 has no hardware interrupts
                ;cld    ;we're not doing any arithmetic beyond DEX/DEY which are unaffected by D
            
                ldx #0
                txa
                ;tay
.CLEAR_STACK    dex
                txs
                pha
                bne .CLEAR_STACK     ; SP=$FF, X = A = 0 (Y inited elsewhere)

MainLoop
    ;overscan (beam off)
    dex
    ;X=$FF
    stx VBLANK
    ldx #OVERSCN-5  ;account for VSYNC
    jsr Burn

    inc frame
    lda frame
    and #$0F
    sta COLUBK      ;could use SAX instead, but I have two bytes to spare

    ;VERTICAL_SYNC
    lda #%1110         ; each '1' bits generate a VSYNC ON line (bits 1..3)
VSLP1
    sta WSYNC           ; 1st '0' bit resets Vsync, 2nd '0' bit exit loop
    sta VSYNC
    lsr
    bne VSLP1           ; branch until VYSNC has been reset

    ;VBLANK
    ldx #VBLNK
    jsr Burn

    ;turn beam on
    stx VBLANK

    ;center vertically
    jsr CenterBurn

    ;draw the text
    ldx #5
Kernel2
    ldy #13
    sty CTRLPF  ;REF (and PFP, but it doesn't matter here)
Kernel
    sta WSYNC
    lda #$08    ;this makes it seem like the text is flashing when it actually isn't
    sta COLUPF

    lda Gfx0-1,X
    sta PF1
    lda Gfx1-1,X
    sta PF2
#if STELLA
    ;Stella does not properly emulate switching reflected PF2 mid-scanline
    ;Maybe z26 handles this correctly?
    SLEEP 22
#else
    ;Tested on a real machine
    SLEEP 21
#endif
    lda Gfx2-1,X
    sta PF2
    lda Gfx3-1,X
    sta PF1
    
    dey
    bpl Kernel
    dex
    bne Kernel2

    ;X=0
    stx PF1
    stx PF2

    ;burn remaining lines
    jsr CenterBurn
    ;X=0
    beq MainLoop

    ;burns half of the remaining lines (called twice)
CenterBurn
    ldx #(LINES-5*14)/2
Burn
    sta WSYNC
    dex
    bne Burn
    rts

    echo "ROM:", ($1FFC - *)d, "bytes left"

    org $1FFC
    .word Start
Delay14
    nop
Delay12
    rts
